@@ -7,63 +7,30 @@
 #ifndef __COMMON_H__
 #define __COMMON_H__
 
-#include "config.h"
+#include <config.h>
 
-/*** Toplevel header files ***/
-#include <sys/param.h>
-#include <assert.h>
-#include <ctype.h>
-#include <dirent.h>
-#include <errno.h>
-#include <fcntl.h>
-#include <gdk-pixbuf/gdk-pixbuf.h>
-#include <glade/glade.h>
-#include <gnome.h>
+#include "datatype.h"
+#include "prefs_string.h"
+#include "prefs.h"
+
+#include <bonobo/bonobo-i18n.h>
+#include <glade/glade-xml.h>
 #include <gtk/gtkwidget.h>
-#include <locale.h>
-#include <netinet/in.h>
-#include <pwd.h>
-#include <signal.h>
-#include <stdarg.h>
-#include <string.h>
-#include <sys/socket.h>
-#include <sys/stat.h>
-#include <sys/types.h>
-#include <sys/socket.h>
-#include <sys/stat.h>
-#include <sys/wait.h>
-#include <termios.h>
-#include <time.h>
-#include <sys/time.h>
-#include <sys/types.h>
-#include <sys/un.h>
-#include <time.h>
-#include <unistd.h>
 
-#ifdef BUILD_APPLET
-#include <applet-widget.h>
-#endif
-
-#ifdef GNOME_ICON
-#include <libgnomeui/gnome-window-icon.h>
-#endif
+#include <time.h>
 #ifdef HAVE_SOCKS5
 #define SOCKS
 #include <socks.h>
 #endif
 
-/*** GnomeICU header files ***/
-#include "datatype.h"
+#undef MIN
+#undef MAX
 
-/* Private data for history window */
-typedef struct _HistoryWindow HistoryWindow;
 
 typedef struct
 {
 	gboolean auto_popup;
 	gboolean full_reply;
-	gboolean applet;
-	gboolean autohide;
 	gboolean connect_startup;
 	gboolean beep;
 	gboolean packet_tcp;
@@ -71,12 +38,10 @@ typedef struct
 	gboolean connection_history;
 	gboolean sound;
 	gboolean webpresence;
-	gboolean show_window;
 	gboolean russian;
 	gboolean latin2;
 	gboolean kanji;
 	gboolean no_new_users;
-	gboolean hidetooltip;
         gboolean status_notify;
 	gboolean sort_contacts;
 	gboolean sound_onlyonline;
@@ -85,7 +50,6 @@ typedef struct
 	gint auto_na;
         gboolean check_spell;
 	gboolean program_on_receive_message;
-	gboolean program_on_chat_request;
 	gboolean program_on_receive_url;
 	gboolean program_on_file_request;
 	gboolean program_on_user_online;
@@ -97,15 +61,14 @@ typedef struct
 	gboolean auto_accept_xfers;
 	gchar *auto_accept_path;
         gchar *theme_name;
+        gchar *emoticon_theme_name;
 	gboolean auto_recon_forced;
-	gboolean show_applet_border;
 	gboolean default_to_chat;
 } _toggles;
 
 typedef struct 
 {
 	gchar *receive_message;
-	gchar *chat_request;
 	gchar *receive_url;
 	gchar *file_request;
 	gchar *user_online;
@@ -116,6 +79,9 @@ typedef struct
 	gchar *contact_list;
 } _programs;
 
+
+#define TCP_VERSION 0x08
+
 #define NOT_SPECIFIED 0
 #define FEMALE 1
 #define MALE 2
@@ -135,6 +101,9 @@ typedef struct
 #define STATUS_OCCUPIED        0x11
 #define STATUS_AWAY            0x01
 #define STATUS_DND             0x13
+#define STATUS_DND_MAC         0x02
+#define STATUS_AIM_ONLINE      0x1000
+#define STATUS_AIM_AWAY        0x3000
 
 #define USER_ADDED_MESS        0x000c
 #define AUTH_REQ_MESS          0x0006
@@ -157,6 +126,7 @@ typedef struct
 	gchar age;       /*General*/
 	gchar sex;       /*General*/
 	guint auth        : 1; /*General*/
+	guint webpresence : 1; /*********/
 	guint hide_email  : 1; /*********/
 	guint birth_day   : 5; /*********/
 	guint birth_month : 4; /*********/
@@ -178,9 +148,9 @@ typedef struct
 	gchar *state;    /*Location*/
 	gchar *zip;       /*Location*/
 	WORD country;    /*Location*/
-	gint c_status;
 	gchar *homepage; /*About*/
 	gchar *about;    /*About*/
+	WORD occupation;     /*Work*/
 	gchar *job_pos;      /*Work*/
 	gchar *department;   /*Work*/
 	gchar *company_name; /*Work*/
@@ -193,47 +163,11 @@ typedef struct
 	gchar *work_homepage;/*Work*/
 	WORD work_country;   /*Work*/
 	GList *emails;	    /*Emails*/
+	GList *homepage_cat; /*Homepage categories*/
+	GList *interests;    /*Interests*/
+	GList *past_background; /*Past & Affiliations*/
+	GList *affiliations; /*Past & Affiliations*/
 
-	GtkWidget *window;
-
-	GtkWidget *w_uin;
-	GtkWidget *w_nick;
-	GtkWidget *w_alias;
-	GtkWidget *w_first;
-	GtkWidget *w_last;
-	GtkWidget *w_sex;
-	GtkWidget *w_age;
-	GtkWidget *w_auth;
-   	GtkWidget *w_birthday;
-
-	GtkWidget *w_ip;
-	GtkWidget *w_port;
-	GtkWidget *w_status;
-	GtkWidget *w_email;
-	GtkWidget *w_hide_email;
-
-	GtkWidget *w_phone;
-	GtkWidget *w_fax;
-	GtkWidget *w_cellular;
-	GtkWidget *w_street;
-	GtkWidget *w_city;
-	GtkWidget *w_state;
-	GtkWidget *w_zip;
-	GtkWidget *w_country;
-	GtkWidget *w_c_status;
-
-	GtkWidget *w_homepage;
-	GtkWidget *w_about;
-
-	GtkWidget *w_work_city;
-	GtkWidget *w_work_state;
-	GtkWidget *w_work_phone;
-	GtkWidget *w_work_fax;
-	GtkWidget *w_work_address;
-	GtkWidget *w_work_homepage;
-	GtkWidget *w_company_name;
-	GtkWidget *w_department;
-	GtkWidget *w_job_pos;
 } USER_INFO_STRUCT, *USER_INFO_PTR;
 
 typedef enum
@@ -254,58 +188,59 @@ typedef struct
 	time_t time;
 	gchar *message;
 	gpointer data;       /* Message type dependent data */
-	
-	gchar *chatsessionname;
 } stored_message, *STORED_MESSAGE_PTR;
 
+typedef enum {
+  NO, YES,
+  OUTGOING1, OUTGOING2,
+  INCOMING
+} tcp_conn;
+
 typedef struct
 {
 	UIN_T uin;
 	DWORD status;
-	DWORD last_status;
 	DWORD lb_index;
 	DWORD last_time; /* last time online or when came online */
-        gboolean has_direct_connect;
+	gboolean has_rtf;
+	gboolean has_direct_connect;
 	DWORD current_ip;
 	DWORD port;
-        WORD  version;
+	DWORD direct_cookie;
+	WORD  version;
+	WORD  used_version;
 	int sok;
-	gchar nick[20];
+	gchar* nick;
 	USER_INFO_PTR info;
 	GSList *stored_messages;
-        GList *chat_requests;
 	guint invis_list     : 1;
-	guint vis_list       : 1;
+	guint vis_list	     : 1;
 	guint ignore_list    : 1;
-	guint force_override : 1;
-	guint need_update    : 1;
-	guint show_again     : 1;
 	guint online_notify  : 1;
-	guint inlist         : 1;
-	guchar force;
-	int info_seq;
+	guint inlist	     : 1;
+	gboolean newmsg_blink : 1;
 	GSList *tcp_msg_queue;
-	guint tcp_seq;
-	gboolean have_tcp_connection;
+	WORD tcp_seq;
+	tcp_conn have_tcp_connection;
 	WORD tcp_buf_len;
 	BYTE *tcp_buf;
 	WORD tcp_buf_read;
 	GIOChannel *gioc;
 	guint giocw;
 	GtkWidget *read_next;
-	GdkPixmap *icon_p;
-	GdkBitmap *icon_b;
-	GtkWidget *message_chat;
 	GtkWidget *detached_window;
-	HistoryWindow *message_history;
+        GladeXML *history_xml;
 	GladeXML *msg_dlg_xml;
 	GladeXML *info_dlg_xml;
 	gboolean confirmed;
-        WORD gid;
-        WORD uid;
-        WORD vislist_uid;
-        WORD invlist_uid;
-        WORD ignorelist_uid;
+	WORD gid;
+	WORD uid;
+	WORD vislist_uid;
+	WORD invlist_uid;
+	WORD ignorelist_uid;
+	gboolean wait_auth; /* whether we need auth grant from this contact */
+	GSList *file_xfers;
+	gint newmsg_blink_timeout;
 } Contact_Member, *CONTACT_PTR;
 
 typedef struct
