@@ -7,21 +7,21 @@
 
 #include <config.h>
 
-#include <glib.h>
-#include <fcntl.h>
-#include <string.h>
-#include <sys/types.h>
-#include <sys/stat.h>
-#include <sys/socket.h>
-#include <sys/un.h>
-#include <stdarg.h>
+#include <bonobo/bonobo-i18n.h>
 #include <errno.h>
-#include <stdio.h>
-#include <stdlib.h>
-#include <unistd.h>
+#include <fcntl.h>
+#include <locale.h>
 #include <netinet/in.h>
 #include <pwd.h>
-
+#include <stdarg.h>
+#include <stdio.h>
+#include <stdlib.h>
+#include <string.h>
+#include <sys/socket.h>
+#include <sys/stat.h>
+#include <sys/types.h>
+#include <sys/un.h>
+#include <unistd.h>
 
 int socket_fd;
 
@@ -37,10 +37,15 @@ int main(int ac,char **av)
 	char *uinstr = NULL;
 	int readv = 0;
 
+	setlocale(LC_ALL, "");
+	
+	bindtextdomain(PACKAGE, GNOMELOCALEDIR);
+	textdomain(PACKAGE);
+
 	if( ac <= 1 || !strcmp( av[1], "-h" ) || !strcmp( av[1], "--help" ) )
 		usage();
 
-	buffer = g_malloc0( 8192 );
+	buffer = malloc( 8192 );
 
 	for(i=1;i<ac;i++) {
 		if(!strcmp(av[i],"-u"))
@@ -77,12 +82,13 @@ int main(int ac,char **av)
 
 	if ( uinstr != NULL && atol(uinstr)==0 )
 	{
-		printf("UIN must be a number.\n");
+		printf("%s", _("UIN must be a number.\n"));
 		return(1);
 	}
 
 	if (connectme(uinstr)) {
-		printf("Unable to connect to running GnomeICU client.\n");
+		printf("%s", 
+		       _("Unable to connect to running GnomeICU client.\n"));
 		return(1);
 	}
 
@@ -119,20 +125,17 @@ int main(int ac,char **av)
 int connectme( char *uinstr )
 {
 	struct sockaddr_un addr;
-	char *user;
 	struct passwd *pw;
 
 	pw = getpwuid(getuid());
 
 	if (!pw) {
-		fprintf(stderr,"Unable to get current login name, naming service down?\n");
+		fprintf(stderr, "%s", _("Unable to get current login name, naming service down?\n"));
 		return(-1);
 	}
 
-	user = strdup( pw->pw_name );
-
 	sprintf(addr.sun_path, "/tmp/.gnomeicu-%s/ctl%s%s",
-	        user,
+	        pw->pw_name,
 	        (uinstr == NULL) ? "" : "-",
 	        (uinstr == NULL) ? "" : uinstr );
 
@@ -158,30 +161,31 @@ int connectme( char *uinstr )
 
 void usage (void)
 {
-	printf ("Usage: icukrell-client [-u YourUIN] command [DestinationUIN] [Data]\n\n");
-	printf ("Commands are as follows:\n");
-	printf ("%s\t\t%s\n", "msgcount", "show number of incoming messages");
-	printf ("%s\t\t\t%s\n", "icuask", "what IcuKrell ask GnomeICU");
-	printf ("%s\t\t\t%s\n", "readmsg", "read message");
-	printf ("%s\t\t\t%s\n", "getstatus", "get GnomeICU status");
-	printf ("%s\t\t\t%s\n", "quit", "close GnomeICU");
-	printf ("%s\t\t\t%s\n", "hide", "hide GnomeICU main window");
-	printf ("%s\t\t\t%s\n", "show", "show GnomeICU main window");
-	printf ("%s\t\t%s\n", "showhide", "show|hide GnomeICU main window");
-	printf ("%s\t\t%s\n", "addcontact", "popup GnomeICU useradd dialog");
-	printf ("%s\t\t%s\n", "connhistory", "popup GnomeICU connection history window");
-	printf ("%s\t\t%s\n", "showprefs", "popup GnomeICU preferencies dialog");
-	printf ("%s\t\t%s\n", "changeinfo", "popup GnomeICU user info dialog");
-	printf ("%s\t\t%s\n", "ignorelist", "popup GnomeICU ignore list");
-	printf ("%s\t\t%s\n", "visiblelist", "popup GnomeICU visible list");
-	printf ("%s\t\t%s\n", "invisiblelist", "popup GnomeICU invisible list");
-	printf ("%s\t\t%s\n", "notifylist", "popup GnomeICU notify list");
-	printf ("%s\t\t%s\n", "away \"msg\"", "set away message");
-	printf ("%s\t%s\n", "adduser|useradd UIN", "add user with UIN to the list");
+	printf ("%s", _("Usage: gnomeicu-client [-u YourUIN] command [DestinationUIN] [Data]\n\n"));
+	printf ("%s", _("Commands are as follows:\n"));
+	printf ("%s\t\t%s\n", "msgcount", _("show number of incoming messages"));
+	printf ("%s\t\t\t%s\n", "icuask", _("what IcuKrell ask GnomeICU"));
+	printf ("%s\t\t\t%s\n", "readmsg", _("read message"));
+	printf ("%s\t\t\t%s\n", "getstatus", _("get GnomeICU status"));
+	printf ("%s\t\t\t%s\n", "quit", _("close GnomeICU"));
+	printf ("%s\t\t\t%s\n", "hide", _("hide GnomeICU main window"));
+	printf ("%s\t\t\t%s\n", "show", _("show GnomeICU main window"));
+	printf ("%s\t\t%s\n", "showhide", _("show|hide GnomeICU main window"));
+	printf ("%s\t\t%s\n", "addcontact", _("popup GnomeICU useradd dialog"));
+	printf ("%s\t\t%s\n", "showprefs", _("popup GnomeICU preferences dialog"));
+	printf ("%s\t\t%s\n", "changeinfo", _("popup GnomeICU user info dialog"));
+	printf ("%s\t\t%s\n", "ignorelist", _("popup GnomeICU ignore list"));
+	printf ("%s\t\t%s\n", "visiblelist",_("popup GnomeICU visible list"));
+	printf ("%s\t\t%s\n", "invisiblelist", _("popup GnomeICU invisible list"));
+	printf ("%s\t\t%s\n", "notifylist", _("popup GnomeICU notify list"));
+	printf ("%s\t\t%s\n", "away \"msg\"", _("set away message"));
+	printf ("%s\t%s\n", "adduser|useradd UIN", _("add user with UIN to the list"));
 	// onlinelist
-	printf ("%s\t\t%s\n", "msg UIN \"msg\"", "send message to UIN");
+	printf ("%s\t\t%s\n", "msg UIN \"msg\"", _("send message to UIN"));
 	printf ("%s\t\t%s\n", "status <status>",
-		"set status: online offline away na invisible freechat occupied dnd");
-	printf ("%s\t\t%s\n", "readevent", "popup GnomeICU current event window");
+		_("set status: online offline away na invisible freechat occupied dnd"));
+	printf ("%s\t%s\n", "status_noask <status>",
+		_("set status without asking the reason: online offline away na invisible freechat occupied dnd"));
+	printf ("%s\t\t%s\n", "readevent", _("popup GnomeICU current event window"));
 	exit (1);
 }
