@@ -3,19 +3,16 @@
 #include "listwindow.h"
 #include "notify.h"
 
+#include <gtk/gtk.h>
+#include <string.h>
+
 static void modify_notify_list( GtkWidget *widget, gpointer data );
 
 static gboolean is_notify (CONTACT_PTR contact);
-static void remove_func (CONTACT_PTR contact);
 
 gboolean is_notify (CONTACT_PTR contact)
 {
-	return contact->online_notify == TRUE;
-}
-
-void remove_func (CONTACT_PTR contact)
-{
-  ;/*do nothing*/
+	return (contact ? contact->online_notify : FALSE);
 }
 
 void notify_list_dialog( void )
@@ -25,22 +22,18 @@ void notify_list_dialog( void )
 	if( dlg == NULL )
 	{
 		dlg = list_window_new(
-			_("GnomeICU: Online Notify List"), "Online_Notify_List",
+			_("Online Notify List"),
 			_("Drag the users you wish to add to your\n"
 			  "notify list into this window, and then click\n"
-			  "the \"Apply\" button below."),
-			GNOME_STOCK_BUTTON_OK, NULL,
-			GTK_SIGNAL_FUNC (modify_notify_list),
-			is_notify,
-                        remove_func
+			  "the \"Ok\" button below."),
+			GTK_STOCK_OK, NULL,
+			G_CALLBACK (modify_notify_list),
+			is_notify
 		);
-		gtk_widget_show_all( dlg );
-		gtk_signal_connect( GTK_OBJECT(dlg), "destroy",
-				    GTK_SIGNAL_FUNC(gtk_widget_destroyed),
-				    &dlg );
+		gtk_widget_show( dlg );
+		g_object_add_weak_pointer (G_OBJECT (dlg), (gpointer *)&dlg);
 	} else {
-		gdk_window_show(dlg->window);
-		gdk_window_raise(dlg->window);
+		gtk_window_present (GTK_WINDOW (dlg));
 	}
 }
 
@@ -70,7 +63,7 @@ void modify_notify_list( GtkWidget *widget, gpointer data )
 		contact = Contacts;
 		while( contact != NULL )
 		{
-			if( atoi( uin ) == kontakt->uin )
+			if( !strcmp(uin, kontakt->uin) )
 			{
 				kontakt->online_notify = TRUE;
 			}
@@ -79,5 +72,5 @@ void modify_notify_list( GtkWidget *widget, gpointer data )
 		}
 	}
 
-	gtk_widget_set_sensitive( widget, FALSE );
+	Save_RC();
 }
