@@ -7,26 +7,24 @@
 #ifndef __RESPONSE_H__
 #define __RESPONSE_H__
 
-#include "chat.h"
 #include "common.h"
+#include "filexfer.h"
+#include "v7base.h"
 
 void User_Online( GSList *contact );
 void User_Offline( GSList *contact );
 void Shortcut_Update( CONTACT_PTR contact );
 void Shortcut_Update_All( void );
-void Display_Search_Reply(WORD reqid, UIN_T uin, gchar *nick, gchar *first,
-                          gchar *last, gchar *email, gchar auth,
-                          gchar status, gchar sex, gchar age);
-ChatContact *Do_Chat( WORD sessionlen, char *data, UIN_T uin, DWORD seq, gchar *chatsession, int port );
-GSList *Do_File( char *data, UIN_T uin, DWORD seq, gchar *file_name, int file_size );
+void Do_File( char *msg, UIN_T uin, gchar *msgid, gchar *file_name,
+		   int file_size, V7Connection *conn);
 void msg_received (UIN_T uin, gchar *msg, time_t msg_time);
 void url_received (UIN_T uin, const gchar *url, const gchar *desc, time_t msg_time);
 void contact_list_received (UIN_T uin, GSList *contacts, time_t msg_time);
 void user_added_you (UIN_T uin, time_t msg_time);
-void authorisation_request(UIN_T uin, gchar *nick, gchar *first, gchar *last,
+void authorization_request(UIN_T uin, gchar *nick, gchar *first, gchar *last,
                            gchar *email, gchar *msg, time_t msg_time);
 void recv_awaymsg( UIN_T uin, DWORD status, const gchar *message);
-
-extern int preset_status;
+void start_transfer( UIN_T uin, gchar *msgid, DWORD fileip, WORD fileport);
+void set_contact_status (GSList *contact, int status);
 
 #endif /* __RESPONSE_H__ */
