@@ -6,11 +6,9 @@
 ***************************/
 
 #include "common.h"
-#include "applet.h"
 #include "changeinfo.h"
 #include "gnomeicu.h"
 #include "gtkfunc.h"
-#include "gtkconf.h"
 #include "icons.h"
 #include "ignore.h"
 #include "invisible.h"
@@ -23,15 +21,38 @@
 #include "util.h"
 #include "visible.h"
 #include "v7send.h"
+#include "showlist.h"
+
+#include "msg.h"
+
+#include <gdk/gdk.h>
+#include <gdk/gdkkeysyms.h>
+#include <errno.h>
+#include <netinet/in.h>
+#include <pwd.h>
+#include <stdio.h>
+#include <stdlib.h>
+#include <string.h>
+#include <sys/types.h>
+#include <sys/socket.h>
+#include <sys/stat.h>
+#include <sys/un.h>
+#include <unistd.h>
+
+static void userserver_handler( gint sock );
+static void show_contact_message_client( GSList *contact, gint sock );
+static void icq_msgbox_client( STORED_MESSAGE_PTR message_text, UIN_T uin,
+			       gint sock );
+static gboolean userserver_accept(GIOChannel *iochan, GIOCondition cond,
+			      gpointer data);
+
 
-void show_contact_message_client( GSList *contact, gint sock );
-void icq_msgbox_client( STORED_MESSAGE_PTR message_text, UIN_T uin, gint sock );
 
 int userserver_socket;
 gchar *userserver_name;
 gchar *userserver_sym;
 
-gint userserver_gdk_input;
+GIOChannel *userserver_iochan;
 
 #define USERSERVER_MAX_LENGTH 8192
 
@@ -129,9 +150,10 @@ void userserver_handler(gint sock)
                         while (contact != NULL)
                         {
 	                        msg_count += g_slist_length(((CONTACT_PTR)contact->data)->stored_messages);
-				if (kontakt->status != STATUS_OFFLINE) online_users++;
 				contact = contact->next;
                         }
+
+			online_users = gnomeicu_tree_online_contacts();
 			
 			sprintf(icu_status, "Unknown");
 			if (Current_Status == STATUS_OFFLINE) sprintf(icu_status, "Offline");
@@ -145,20 +167,22 @@ void userserver_handler(gint sock)
 					sprintf(icu_status, "Invisible");
 					break;
 				case STATUS_NA:
-					sprintf(icu_status, "N/A");
+					sprintf(icu_status, "NotAvailable");
 					break;
 				case STATUS_FREE_CHAT:
-					sprintf(icu_status, "Free4Chat");
+					sprintf(icu_status, "FreeForChat");
 					break;
 				case STATUS_OCCUPIED:
 					sprintf(icu_status, "Occupied");
 					break;
-                    		case STATUS_AWAY:
+				case STATUS_AWAY:
 					sprintf(icu_status, "Away");
 					break;
-                    		case STATUS_DND:
-					sprintf(icu_status, "DND");
+				case STATUS_DND:
+					sprintf(icu_status, "DoNotDisturb");
 		    			break;
+				default:
+					g_warning("Unknown current status: 0x%x\n", Current_Status & 0xffff);
 			}
 			
 			g_free( cmdbuf );
@@ -200,10 +224,10 @@ void userserver_handler(gint sock)
 					sprintf(icu_status, "Invisible");
 					break;
 				case STATUS_NA:
-					sprintf(icu_status, "N/A");
+					sprintf(icu_status, "NotAvailable");
 					break;
 				case STATUS_FREE_CHAT:
-					sprintf(icu_status, "Free4Chat");
+					sprintf(icu_status, "FreeForChat");
 					break;
 				case STATUS_OCCUPIED:
 					sprintf(icu_status, "Occupied");
@@ -212,8 +236,10 @@ void userserver_handler(gint sock)
 					sprintf(icu_status, "Away");
 					break;
                     		case STATUS_DND:
-					sprintf(icu_status, "DND");
+					sprintf(icu_status, "DoNotDisturb");
 		    			break;
+				default:
+					g_warning("Unknown current status: 0x%x\n", Current_Status & 0xffff);
 			}
 
 			g_free( cmdbuf );
@@ -235,39 +261,23 @@ void userserver_handler(gint sock)
 			/* COMMAND: HIDE */
 			else if (!strcasecmp(cmd,"hide"))
 			{
-				if (GTK_WIDGET_VISIBLE(app))
-				{
-				    gtk_widget_hide(app);
-				    if (gnome_win_hints_get_workspace(app) != gnome_win_hints_get_current_workspace())
-					gdk_window_withdraw( GTK_WIDGET(app)->window );
-				}
+				gtk_widget_hide(app);
 			}
 			
 			/* COMMAND: SHOW */
 			else if (!strcasecmp(cmd,"show"))
 			{
-				if (!GTK_WIDGET_VISIBLE(app))
-				{
-				    if (gnome_win_hints_get_workspace(app) != gnome_win_hints_get_current_workspace())
-					gnome_win_hints_set_workspace(app, gnome_win_hints_get_current_workspace());
-				    gtk_widget_show(app);
-				}    
+				gtk_window_present (GTK_WINDOW(app));
 			}
 			
 			/* COMMAND: SHOWHIDE */
 			else if (!strcasecmp(cmd,"showhide"))
 			{
-				if (GTK_WIDGET_VISIBLE(app))
-				{
-				    gtk_widget_hide(app);
-				    if (gnome_win_hints_get_workspace(app) != gnome_win_hints_get_current_workspace())
-					gdk_window_withdraw( GTK_WIDGET(app)->window );
-				} else
-				{
-				    if (gnome_win_hints_get_workspace(app) != gnome_win_hints_get_current_workspace())
-					gnome_win_hints_set_workspace(app, gnome_win_hints_get_current_workspace());
-				    gtk_widget_show(app);
-				}    
+				if (GTK_WIDGET_VISIBLE (app)) {
+					gtk_widget_hide (app);
+				} else {
+					gtk_window_present (GTK_WINDOW(app));
+				}
 			}
 			
 			/* COMMAND: ADDCONTACT */
@@ -276,7 +286,7 @@ void userserver_handler(gint sock)
 				
 			/* COMMAND: SHOWPREFS */
 			else if (!strcasecmp(cmd,"showprefs"))
-				configure_window(NULL, 0);
+                          preferences_dialog_show ();
 			
 			/* COMMAND: ISERINFO */
 			else if (!strcasecmp(cmd,"changeinfo"))
@@ -319,7 +329,9 @@ void userserver_handler(gint sock)
 				uin=strtok(NULL," ");
 				if (!uin) break;
 	
-				Add_User(atol(uin), uin, TRUE);
+				/* FIXME for groups, etc..
+				Add_User(uin, NULL, TRUE);
+				*/
 			}
 
 			/* COMMAND: ONLINELIST */
@@ -332,7 +344,7 @@ void userserver_handler(gint sock)
                     	    {
 	                        if (kontakt->status != STATUS_OFFLINE)
 				{
-				    sprintf(online_list_, "%s\t%d\t", kontakt->nick, kontakt->uin);
+				    sprintf(online_list_, "%s\t%s\t", kontakt->nick, kontakt->uin);
 				    strcat(online_list, online_list_);
 				    switch (kontakt->status & 0xffff)
 				    {
@@ -343,26 +355,22 @@ void userserver_handler(gint sock)
 					    strcat(online_list, "Invisible");
 					    break;
 					case STATUS_NA:
-					    strcat(online_list, "N/A");
-					    break;
-					case STATUS_NA_99A:
-					    strcat(online_list, "N/A");
+					    strcat(online_list, "NotAvailable");
 					    break;
 					case STATUS_FREE_CHAT:
-					    strcat(online_list, "Free4Chat");
+					    strcat(online_list, "FreeForChat");
 					    break;
 					case STATUS_OCCUPIED:
 					    strcat(online_list, "Occupied");
 					    break;
-					case STATUS_OCCUPIED_MAC:
-					    strcat(online_list, "Occupied");
-					    break;
 					case STATUS_AWAY:
 					    strcat(online_list, "Away");
 					    break;
 					case STATUS_DND:
-					    strcat(online_list, "DND");
+					    strcat(online_list, "DoNotDisturb");
 					    break;
+					default:
+						g_warning("Unknown status: 0x%x\n", kontakt->status & 0xffff);
 				    }
 				    strcat(online_list, "\t");
 				}
@@ -393,15 +401,12 @@ void userserver_handler(gint sock)
 
 			    while( contact != NULL )
 			    {
-				if (kontakt->uin == iuin)
+				if (!strcmp(kontakt->uin, newuin))
 				{
-				    GdkEventButton *event;
-				
-				    event = g_new0( GdkEventButton, 1 );
-				    event->type = GDK_KEY_PRESS;
-				    ((GdkEventKey*)event)->keyval = GDK_Return;
-				    
-				    icq_sendmessage_window(MainData->lb_userwin, event, contact);
+					if (g_slist_length (kontakt->stored_messages))
+						show_contact_message (contact);
+					else
+						open_message_dlg_with_message (contact, 0, NULL);
 				}
 			    
 				contact = contact->next;
@@ -421,7 +426,7 @@ void userserver_handler(gint sock)
 				msg=strtok(NULL,"");
 				if (!msg) break;
 
-                                v7_sendmsg(atoi(uin), msg);
+                                v7_sendmsg(mainconnection, uin, msg);
 				/*  icq_sendmsg(atoi(uin),msg,TRUE); */
 				break;
 			}
@@ -450,6 +455,32 @@ void userserver_handler(gint sock)
 					icq_set_status_dnd( NULL, NULL );
 			}
 
+
+			/* COMMAND: STATUS_NOASK <offline|online|na|invisible|freechat|occupied|away|dnd> */
+			else if (!strcasecmp(cmd,"status_noask"))
+			{
+				args=strtok(NULL," \r\n");
+				if (!args) break;
+
+				if (!strcasecmp(args,"offline"))
+					icq_set_status_offline( NULL, NULL );
+				if (!strcasecmp(args,"online"))
+					icq_set_status_online( NULL, NULL );
+				if (!strcasecmp(args,"na"))
+					icq_set_status_na( NULL, (gpointer)1 );
+				if (!strcasecmp(args,"invisible"))
+					icq_set_status_invisible( NULL, NULL );
+				if (!strcasecmp(args,"freechat"))
+					icq_set_status_ffc( NULL, NULL );
+				if (!strcasecmp(args,"occupied"))
+					icq_set_status_occ( NULL, (gpointer)1 );
+				if (!strcasecmp(args,"away"))
+					icq_set_status_away( NULL, (gpointer)1 );
+				if (!strcasecmp(args,"dnd"))
+					icq_set_status_dnd( NULL, (gpointer)1 );
+			}
+
+
 			/* COMMAND: READEVENT */
 			else if (!strcasecmp(cmd,"readevent"))
 			{
@@ -462,6 +493,8 @@ void userserver_handler(gint sock)
 					show_contact_message( contact );
 			}
 			break;
+		default:
+			g_warning("Unknown command type: %d\n", cmdtype);
 	}
 
 	free(cmdbuf);
@@ -469,9 +502,10 @@ void userserver_handler(gint sock)
 }
 
 
-void userserver_accept(gpointer data,gint sock,GdkInputCondition cond)
+gboolean userserver_accept(GIOChannel *iochan, GIOCondition cond, gpointer data)
 {
 	int fd;
+	int sock = g_io_channel_unix_get_fd(iochan);
 
 	fd = accept(sock, NULL,NULL);
 
@@ -481,6 +515,7 @@ void userserver_accept(gpointer data,gint sock,GdkInputCondition cond)
 		close(fd);
 	}
 
+	return TRUE;
 }
 
 /* This code is borrowed heavily (read: cut&paste) from sawmill.
@@ -488,10 +523,7 @@ void userserver_accept(gpointer data,gint sock,GdkInputCondition cond)
  */
 void userserver_init(void)
 {
-	gchar namebuf[1024];
-	gchar *user;
-	gchar *uinstr;
-	gchar *sym;
+        gchar *directory;
 
 	struct passwd *pw;
 
@@ -507,64 +539,45 @@ void userserver_init(void)
 	    return;
 	  }
 
+	directory = g_strdup_printf("/tmp/.gnomeicu-%s", pw->pw_name);
 
-	user = g_strdup( pw->pw_name );
-
-	if (!user)
-	{
-	  fprintf(stderr,"No memory to duplicate username.\n");
-	  return;
-	}
-
-	sprintf(namebuf,"/tmp/.gnomeicu-%s",user);
-
-	g_free( user );
-	
-	if (mkdir(namebuf,0700) != 0)
+	if (mkdir(directory,0700) != 0)
 	{
 		if (errno == EEXIST)
 		{
 			struct stat st;
 
-			if (stat(namebuf,&st) == 0)
+			if (stat(directory,&st) == 0)
 			{
 				if ((st.st_uid != getuid()) || (st.st_mode & (S_IRWXG | S_IRWXO)))
 				{
 					fprintf(stderr,"The %s directory is not owned by uid %d, or permissions are not secure enough.\n",
-					        namebuf,getuid());
+					        directory,getuid());
 				}
 			} else {
-				perror(namebuf);
+				perror(directory);
 				return;
 			}
 		} else {
-			perror(namebuf);
+			perror(directory);
 			return;
 		}
 	}
-	strcat(namebuf, "/ctl" );
-	sym = g_strdup( namebuf );
-	userserver_sym = g_strdup( namebuf );
-	strcat(namebuf,"-");
+	userserver_sym = g_strdup_printf("%s/ctl", directory);
+	userserver_name = g_strdup_printf("%s/ctl-%s", directory, our_info->uin);
 
-	uinstr = g_strdup_printf( "%d", our_info->uin );
+	symlink( userserver_name, userserver_sym );
 
-	strcat(namebuf,uinstr);
-
-	symlink( namebuf, sym );
-
-	g_free( sym );
-	g_free( uinstr );
 	
 	/* Delete the socket if it exists */
-	if(access(namebuf, F_OK) == 0)
+	if(access(userserver_name, F_OK) == 0)
 	{
 		/* Socket already exists. Delete it */
-		unlink(namebuf);
+		unlink(userserver_name);
 			
-		if (access (namebuf, F_OK) == 0)
+		if (access (userserver_name, F_OK) == 0)
 		{
-			fprintf (stderr, "Can't delete %s\n", namebuf);
+			fprintf (stderr, "Can't delete %s\n", userserver_name);
 			return;
 		}
 	}
@@ -576,25 +589,24 @@ void userserver_init(void)
 		struct sockaddr_un addr;
 
 		addr.sun_family = AF_UNIX;
-		strcpy(addr.sun_path, namebuf);
+		strcpy(addr.sun_path, userserver_name);
 		
 		if(bind(userserver_socket, (struct sockaddr *)&addr,
 			sizeof(addr.sun_family) + strlen(addr.sun_path) + 1) == 0)
 		{
-			chmod (namebuf, 0700);
+			chmod (userserver_name, 0700);
 
 			if( listen(userserver_socket, 5) == 0 )
 			{
-				userserver_gdk_input = gdk_input_add( userserver_socket,
-								GDK_INPUT_READ|GDK_INPUT_EXCEPTION,
-								(GdkInputFunction)userserver_accept,
-								NULL);
-				userserver_name = g_strdup(namebuf);
-				return;
+			  userserver_iochan = g_io_channel_unix_new(userserver_socket);
+			  g_io_add_watch(userserver_iochan,
+					 G_IO_IN|G_IO_ERR|G_IO_NVAL,
+					 userserver_accept, NULL);
+			  return;
 			} else {
 				perror ("listen");
 				close(userserver_socket);
-				unlink(namebuf);
+				unlink(userserver_name);
 			}
 		} else {
 			perror ("bind");
@@ -604,6 +616,8 @@ void userserver_init(void)
 		perror ("socket");
 
 	userserver_socket = -1;
+	g_free(userserver_name);
+	userserver_name = NULL;
 }
 
 void userserver_kill (void)
@@ -617,7 +631,7 @@ void userserver_kill (void)
 		tmp[size] = 0;
 		if( !strcmp( userserver_name, tmp ) )
 			unlink( userserver_sym );
-		gdk_input_remove( userserver_gdk_input );
+		g_io_channel_shutdown(userserver_iochan, TRUE, NULL);
 		close(userserver_socket);
 		userserver_socket = -1;
 		unlink(userserver_name);
@@ -628,106 +642,86 @@ void userserver_kill (void)
 
 void show_contact_message_client( GSList *contact, gint sock )
 {
-	STORED_MESSAGE_PTR msg = (STORED_MESSAGE_PTR)kontakt->stored_messages->data;
+	STORED_MESSAGE_PTR msg;
 
+	g_assert(contact != NULL);
+	msg = (STORED_MESSAGE_PTR)kontakt->stored_messages->data;
 	if( kontakt->read_next &&
 	    GTK_IS_WIDGET( kontakt->read_next ) &&
 	    GTK_WIDGET_VISIBLE( kontakt->read_next ) )
-		gtk_signal_emit_by_name( GTK_OBJECT( kontakt->read_next ), "clicked", MainData );
+		read_next (contact);
 
 	if( g_slist_length( kontakt->stored_messages ) )
 	{
 		icq_msgbox_client( (msg), kontakt->uin, sock );
-		g_free(msg->chatsessionname);
-		if (msg->type == MESSAGE_CHAT_REQ && ((ChatContact *)msg->data)->contact == NULL)
-		     /* FIXME: free members of ChatContact */
-		     g_free(msg->data);
 		g_free( msg->message );
 		g_free( kontakt->stored_messages->data );
 		kontakt->stored_messages = g_slist_remove( kontakt->stored_messages, kontakt->stored_messages->data );
 
-		if( g_slist_length( kontakt->stored_messages ) )
-		{
-			kontakt->icon_p = get_pixmap_for_status( kontakt->status );
-			kontakt->icon_b = get_bitmap_for_status( kontakt->status );
-			kontakt->need_update = 1;
-		}
-
-		Update_Contact_Style( Which_List( kontakt ), kontakt );
-		applet_update( Current_Status, FALSE );
 		Shortcut_Update( kontakt );
 	}
 }
 
 void icq_msgbox_client( STORED_MESSAGE_PTR message_text, UIN_T uin, gint sock )
 {
-	gint type = message_text->type;
-	gchar *text = message_text->message;
+	gint type;
+	gchar *text;
 	gchar *retmsg;
 	GSList *contact;
 	unsigned long u;
 
+	g_assert(message_text != NULL);
+
 	contact = Find_User( uin );
 	if( contact == NULL )
 		return;
 
-	if( g_slist_length( kontakt->stored_messages ) == 1 )
-	{
-		kontakt->icon_p = get_pixmap_for_status( kontakt->status );
-		kontakt->icon_b = get_bitmap_for_status( kontakt->status );
-		kontakt->need_update = 1;
-	}
-
+	type = message_text->type;
+	text = message_text->message;
+	
 	switch( type )
 	{
 		case MESSAGE_TEXT:
-			retmsg = g_strdup_printf( "Message from %s (%d):\n%s\n",
+			retmsg = g_strdup_printf( "Message from %s (%s):\n%s\n",
 			                          kontakt->nick,
 			                          kontakt->uin,
 			                          text );
 			break;
 		case MESSAGE_URL:
-			retmsg = g_strdup_printf( "URL from %s (%d):\n%s\n%s\n",
+			retmsg = g_strdup_printf( "URL from %s (%s):\n%s\n%s\n",
 			                          kontakt->nick,
 			                          kontakt->uin,
 			                          (char *)message_text->data,
 			                          text );
 			break;
 		case MESSAGE_FILE_REQ:
-			retmsg = g_strdup_printf( "File request from %s (%d):\nAuto refusing",
+			retmsg = g_strdup_printf( "File request from %s (%s):\nAuto refusing",
 			                          kontakt->nick,
 			                          kontakt->uin );
-			TCPRefuseFile( message_text->data );
-			break;
-		case MESSAGE_CHAT_REQ:
-			retmsg = g_strdup_printf( "Chat request from %s (%d):\nAuto refusing",
-			                          kontakt->nick,
-			                          kontakt->uin );
-			TCPRefuseChat( kontakt->sok,
-				       (ChatContact *)((STORED_MESSAGE_PTR)message_text)->data );
+			TCPRefuseFile( message_text->data ); 
 			break;
 		case MESSAGE_AUTH_REQ:
-			retmsg = g_strdup_printf( "Authorization request from %s (%d):\nAuto refusing",
+			retmsg = g_strdup_printf( "Authorization request from %s (%s):\nAuto refusing",
 			                          kontakt->nick,
 			                          kontakt->uin );
 			break;
 		case MESSAGE_USER_ADD:
-			retmsg = g_strdup_printf( "%s (%d) has added you to his/her list.\n",
+			retmsg = g_strdup_printf( "%s (%s) has added you to his/her list.\n",
 			                          kontakt->nick,
 			                          kontakt->uin );
 			break;
 		case MESSAGE_USER_AUTHD:
-			retmsg = g_strdup_printf( "%s (%d) has authorized you to add him/her to your list.\n",
+			retmsg = g_strdup_printf( "%s (%s) has authorized you to add him/her to your list.\n",
 			                          kontakt->nick,
 			                          kontakt->uin );
 			break;
 		case MESSAGE_CONT_LIST:
-			retmsg = g_strdup_printf( "%s (%d) sent you a contact list.\n",
+			retmsg = g_strdup_printf( "%s (%s) sent you a contact list.\n",
 			                          kontakt->nick,
 			                          kontakt->uin );
 			break;
 		default:
-			retmsg = g_strdup_printf( "Unknown message from %s (%d).\n",
+			retmsg = g_strdup_printf( "Unknown message from %s (%s).\n",
 			                          kontakt->nick,
 			                          kontakt->uin );
 			break;
@@ -736,4 +730,5 @@ void icq_msgbox_client( STORED_MESSAGE_PTR message_text, UIN_T uin, gint sock )
 	u = g_htonl( strlen( retmsg ) + 1 );
 	write( sock, &u, sizeof( unsigned long ) );
 	write( sock, retmsg, strlen( retmsg ) + 1 );
+	g_free(retmsg);
 }
