@@ -1,24 +1,29 @@
 /*******************************
  General utility functions
- (c) 1999 Jeremy Wise
+ (c) 2002 Jeremy Wise
  GnomeICU
 ********************************/
 
 #include "common.h"
 #include "gnomecfg.h"
 #include "gnomeicu.h"
+#include "gtkfunc.h"
 #include "icons.h"
-#include "showlist.h"
+#include "newsignup.h"
 #include "util.h"
 #include "v7send.h"
 
-typedef struct 
+#include <libgnome/gnome-config.h>
+
+#include <string.h>
+
+typedef struct
 {
   const char *name;
   WORD code;
-} COUNTRY_CODE;
+} FIELD_CODE;
 
-static COUNTRY_CODE Country_Codes[] = {
+static FIELD_CODE Country_Codes[] = {
   { N_("Afghanistan"), 93 },
   { N_("Albania"), 355 },
   { N_("Algeria"), 213 },
@@ -262,69 +267,255 @@ static COUNTRY_CODE Country_Codes[] = {
   { N_("Zambia"), 260 },
   { N_("Zimbabwe"), 263 },
   { N_("Not entered"), 0 },
-  { N_("Not entered"), 0xffff } 
+  { N_("Unknown"), 0xffff } 
 };
 
-typedef struct
-{
-	const char *str;
-	int gmt_offset;
-} TIME_ZONE;
-
-TIME_ZONE time_zones[] =
+/* the code is equivalent to GMT offset */
+static FIELD_CODE time_zones[] =
 {
   /* { N_("UTC+1300 (Tonga)"), 1300 }, Unused */
-{ N_("UTC+1200 (Fiji)"), -24 },
-{ N_("UTC+1130"), -23 },
-{ N_("UTC+1100 (Solomon Islands)"), -22 },
-{ N_("UTC+1030"), -21 },
-{ N_("UTC+1000 (Guam)"), -20 },
-{ N_("UTC+0930"), -19 },
-{ N_("UTC+0900 (Japan)"), -18 },
-{ N_("UTC+0830"), -17 },
-{ N_("UTC+0800 (China/Taiwan)"), -16 },
-{ N_("UTC+0730"), -15 },
-{ N_("UTC+0700 (Cambodia)"), -14 },
-{ N_("UTC+0630"), -13 },
-{ N_("UTC+0600 (Bangladesh)"), -12 },
-{ N_("UTC+0530"), -11 },
-{ N_("UTC+0500 (Turkmenistan)"), -10 },
-{ N_("UTC+0430"), -9 },
-{ N_("UTC+0400 (Yemen/Oman/Armenia)"), -8 },
-{ N_("UTC+0330"), -7 },
-{ N_("UTC+0300 (Eastern African)"), -6 },
-{ N_("UTC+0230"), -5 },
-{ N_("UTC+0200 (Eastern European)"), -4 },
-{ N_("UTC+0130"), -3 },
-{ N_("UTC+0100 (Middle European)"), -2 },
-{ N_("UTC+0030"), -1 },
+{ N_("UTC+1200 (Fiji)"), -24 & 0xff },
+{ N_("UTC+1130"), -23 & 0xff },
+{ N_("UTC+1100 (Solomon Islands)"), -22 & 0xff },
+{ N_("UTC+1030"), -21 & 0xff },
+{ N_("UTC+1000 (Guam)"), -20 & 0xff },
+{ N_("UTC+0930"), -19 & 0xff },
+{ N_("UTC+0900 (Japan)"), -18 & 0xff },
+{ N_("UTC+0830"), -17 & 0xff },
+{ N_("UTC+0800 (China/Taiwan)"), -16 & 0xff },
+{ N_("UTC+0730"), -15 & 0xff },
+{ N_("UTC+0700 (Cambodia)"), -14 & 0xff },
+{ N_("UTC+0630"), -13 & 0xff },
+{ N_("UTC+0600 (Bangladesh)"), -12 & 0xff },
+{ N_("UTC+0530"), -11 & 0xff },
+{ N_("UTC+0500 (Turkmenistan)"), -10 & 0xff },
+{ N_("UTC+0430"), -9 & 0xff },
+{ N_("UTC+0400 (Yemen/Oman/Armenia)"), -8 & 0xff },
+{ N_("UTC+0330"), -7 & 0xff },
+{ N_("UTC+0300 (Eastern African)"), -6 & 0xff },
+{ N_("UTC+0230"), -5 & 0xff },
+{ N_("UTC+0200 (Eastern European)"), -4 & 0xff },
+{ N_("UTC+0130"), -3 & 0xff },
+{ N_("UTC+0100 (Middle European)"), -2 & 0xff },
+{ N_("UTC+0030"), -1 & 0xff },
 { N_("UTC"), 0 },
-{ N_("UTC-0030"), 1 },
-{ N_("UTC-0100"), 2 },
-{ N_("UTC-0130"), 3 },
-{ N_("UTC-0200 (Central African)"), 4 },
-{ N_("UTC-0230"), 5 },
-{ N_("UTC-0300 (Brazil)"), 6 },
-{ N_("UTC-0330"), 7 },
-{ N_("UTC-0400 (Atlantic)"), 8 },
-{ N_("UTC-0430"), 9 },
-{ N_("UTC-0500 (Eastern)"), 10 },
-{ N_("UTC-0530"), 11 },
-{ N_("UTC-0600 (Central)"), 12 },
-{ N_("UTC-0630"), 13 },
-{ N_("UTC-0700 (Mountain)"), 14 },
-{ N_("UTC-0730"), 15 },
-{ N_("UTC-0800 (Pacific)"), 16 },
-{ N_("UTC-0830"), 17 },
-{ N_("UTC-0900 (Alaska)"), 18 },
-{ N_("UTC-0930"), 19 },
-{ N_("UTC-1000 (Hawaii/Aleutian Islands)"), 20 },
-{ N_("UTC-1030"), 21 },
-{ N_("UTC-1100 (American Samoa)"), 22 },
-{ N_("UTC-1130"), 23 },
-{ N_("UTC-1200"), 24 },
-{ N_("UTC-1230"), 25 },
-{ N_("Unknown"), 100 }
+{ N_("UTC-0030"), 1 & 0xff },
+{ N_("UTC-0100"), 2 & 0xff },
+{ N_("UTC-0130"), 3 & 0xff },
+{ N_("UTC-0200 (Central African)"), 4 & 0xff },
+{ N_("UTC-0230"), 5 & 0xff },
+{ N_("UTC-0300 (Brazil)"), 6 & 0xff },
+{ N_("UTC-0330"), 7 & 0xff },
+{ N_("UTC-0400 (Atlantic)"), 8 & 0xff },
+{ N_("UTC-0430"), 9 & 0xff },
+{ N_("UTC-0500 (Eastern)"), 10 & 0xff },
+{ N_("UTC-0530"), 11 & 0xff },
+{ N_("UTC-0600 (Central)"), 12 & 0xff },
+{ N_("UTC-0630"), 13 & 0xff },
+{ N_("UTC-0700 (Mountain)"), 14 & 0xff },
+{ N_("UTC-0730"), 15 & 0xff },
+{ N_("UTC-0800 (Pacific)"), 16 & 0xff },
+{ N_("UTC-0830"), 17 & 0xff },
+{ N_("UTC-0900 (Alaska)"), 18 & 0xff },
+{ N_("UTC-0930"), 19 & 0xff },
+{ N_("UTC-1000 (Hawaii/Aleutian Islands)"), 20 & 0xff },
+{ N_("UTC-1030"), 21 & 0xff },
+{ N_("UTC-1100 (American Samoa)"), 22 & 0xff },
+{ N_("UTC-1130"), 23 & 0xff },
+{ N_("UTC-1200"), 24 & 0xff },
+{ N_("UTC-1230"), 25 & 0xff },
+{ N_("Unknown"), 100 & 0xff }
+};
+
+static FIELD_CODE interest_codes[] =
+{
+	{ N_("Art"), 100 },
+	{ N_("Cars"), 101 },
+	{ N_("Celebrity Fans"), 102 },
+	{ N_("Collections"), 103 },
+	{ N_("Computers"), 104 },
+	{ N_("Culture & Literature"), 105 },
+	{ N_("Fitness"), 106 },
+	{ N_("Games"), 107 },
+	{ N_("Hobbies"), 108 },
+	{ N_("ICQ - Providing Help"), 109 },
+	{ N_("Internet"), 110 },
+	{ N_("Lifestyle"), 111 },
+	{ N_("Movies/TV"), 112 },
+	{ N_("Music"), 113 },
+	{ N_("Outdoor Activities"), 114 },
+	{ N_("Parenting"), 115 },
+	{ N_("Pets/Animals"), 116 },
+	{ N_("Religion"), 117 },
+	{ N_("Science/Technology"), 118 },
+	{ N_("Skills"), 119 },
+	{ N_("Sports"), 120 },
+	{ N_("Web Design"), 121 },
+	{ N_("Nature and Environment"), 122 },
+	{ N_("News & Media"), 123 },
+	{ N_("Government"), 124 },
+	{ N_("Business & Economy"), 125 },
+	{ N_("Mystics"), 126 },
+	{ N_("Travel"), 127 },
+	{ N_("Astronomy"), 128 },
+	{ N_("Space"), 129 },
+	{ N_("Clothing"), 130 },
+	{ N_("Parties"), 131 },
+	{ N_("Women"), 132 },
+	{ N_("Social science"), 133 },
+	{ N_("60's"), 134 },
+	{ N_("70's"), 135 },
+	{ N_("80's"), 136 },
+	{ N_("50's"), 137 },
+	{ N_("Finance and Corporate"), 138 },
+	{ N_("Entertainment"), 139 },
+	{ N_("Consumer Electronics"), 140 },
+	{ N_("Retail Stores"), 141 },
+	{ N_("Health and Beauty"), 142 },
+	{ N_("Media"), 143 },
+	{ N_("Household Products"), 144 },
+	{ N_("Mail Order Catalog"), 145 },
+	{ N_("Business Services"), 146 },
+	{ N_("Audio and Visual"), 147 },
+	{ N_("Sporting and Athletic"), 148 },
+	{ N_("Publishing"), 149 },
+	{ N_("Home Automation"), 150 },
+	{ N_("Unknown"), 0xff } 
+};
+
+static FIELD_CODE language_codes[] =
+{
+	{ N_("Arabic"), 1 },
+	{ N_("Bhojpuri"), 2 },
+	{ N_("Bulgarian"), 3 },
+	{ N_("Burmese"), 4 },
+	{ N_("Cantonese"), 5 },
+	{ N_("Catalan"), 6 },
+	{ N_("Chinese"), 7 },
+	{ N_("Croatian"), 8 },
+	{ N_("Czech"), 9 },
+	{ N_("Danish"), 10 },
+	{ N_("Dutch"), 11 },
+	{ N_("English"), 12 },
+	{ N_("Esperanto"), 13 },
+	{ N_("Estonian"), 14 },
+	{ N_("Farci"), 15 },
+	{ N_("Finnish"), 16 },
+	{ N_("French"), 17 },
+	{ N_("Gaelic"), 18 },
+	{ N_("German"), 19 },
+	{ N_("Greek"), 20 },
+	{ N_("Hebrew"), 21 },
+	{ N_("Hindi"), 22 },
+	{ N_("Hungarian"), 23 },
+	{ N_("Icelandic"), 24 },
+	{ N_("Indonesian"), 25 },
+	{ N_("Italian"), 26 },
+	{ N_("Japanese"), 27 },
+	{ N_("Khmer"), 28 },
+	{ N_("Korean"), 29 },
+	{ N_("Lao"), 30 },
+	{ N_("Latvian"), 31 },
+	{ N_("Lithuanian"), 32 },
+	{ N_("Malay"), 33 },
+	{ N_("Norwegian"), 34 },
+	{ N_("Polish"), 35 },
+	{ N_("Portuguese"), 36 },
+	{ N_("Romanian"), 37 },
+	{ N_("Russian"), 38 },
+	{ N_("Serbo-Croatian"), 39 },
+	{ N_("Slovak"), 40 },
+	{ N_("Slovenian"), 41 },
+	{ N_("Somali"), 42 },
+	{ N_("Spanish"), 43 },
+	{ N_("Swahili"), 44 },
+	{ N_("Swedish"), 45 },
+	{ N_("Tagalog"), 46 },
+	{ N_("Tatar"), 47 },
+	{ N_("Thai"), 48 },
+	{ N_("Turkish"), 49 },
+	{ N_("Ukrainian"), 50 },
+	{ N_("Urdu"), 51 },
+	{ N_("Vietnamese"), 52 },
+	{ N_("Yiddish"), 53 },
+	{ N_("Yoruba"), 54 },
+	{ N_("Afrikaans"), 55 },
+	{ N_("Bosnian"), 56 },
+	{ N_("Persian"), 57 },
+	{ N_("Albanian"), 58 },
+	{ N_("Not entered"), 0 },
+	{ N_("Unknown"), 0xff } 
+};
+
+static FIELD_CODE past_codes[] =
+{
+	{ N_("Elementary School"), 300 },
+	{ N_("High School"), 301 },
+	{ N_("College"), 302 },
+	{ N_("University"), 303 },
+	{ N_("Military"), 304 },
+	{ N_("Past Work Place"), 305 },
+	{ N_("Past Organization"), 306 },
+	{ N_("Other"), 399 },
+	{ N_("Unknown"), 0xffff } 
+};
+
+static FIELD_CODE work_codes[] =
+{
+	{ N_("Academic"), 1 },
+	{ N_("Administrative"), 2 },
+	{ N_("Art/Entertainment"), 3 },
+	{ N_("College Student"), 4 },
+	{ N_("Computers"), 5 },
+	{ N_("Community & Social"), 6 },
+	{ N_("Education"), 7 },
+	{ N_("Engineering"), 8 },
+	{ N_("Financial Services"), 9 },
+	{ N_("Government"), 10 },
+	{ N_("High School Student"), 11 },
+	{ N_("Home"), 12 },
+	{ N_("ICQ - Providing Help"), 13 },
+	{ N_("Law"), 14 },
+	{ N_("Managerial"), 15 },
+	{ N_("Manufacturing"), 16 },
+	{ N_("Medical/Health"), 17 },
+	{ N_("Military"), 18 },
+	{ N_("Non-Government Organization"), 19 },
+	{ N_("Professional"), 20 },
+	{ N_("Retail"), 21 },
+	{ N_("Retired"), 22 },
+	{ N_("Science & Research"), 23 },
+	{ N_("Sports"), 24 },
+	{ N_("Technical"), 25 },
+	{ N_("University Student"), 26 },
+	{ N_("Web Building"), 27 },
+	{ N_("Other Services"), 99 },
+	{ N_("Unknown"), 0xffff } 
+};
+
+static FIELD_CODE affiliation_codes[] =
+{
+	{ N_("Alumni Org."), 200 },
+	{ N_("Charity Org."), 201 },
+	{ N_("Club/Social Org."), 202 },
+	{ N_("Community Org."), 203 },
+	{ N_("Cultural Org."), 204 },
+	{ N_("Fan Clubs"), 205 },
+	{ N_("Fraternity/Sorority"), 206 },
+	{ N_("Hobbyists Org."), 207 },
+	{ N_("International Org."), 208 },
+	{ N_("Nature and Environment Org."), 209 },
+	{ N_("Professional Org."), 210 },
+	{ N_("Scientific/Technical Org."), 211 },
+	{ N_("Self Improvement Group"), 212 },
+	{ N_("Spiritual/Religious Org."), 213 },
+	{ N_("Sports Org."), 214 },
+	{ N_("Support Org."), 215 },
+	{ N_("Trade and Business Org."), 216 },
+	{ N_("Union"), 217 },
+	{ N_("Voluntary Org."), 218 },
+	{ N_("Other"), 299 },
+	{ N_("Unknown"), 0xffff } 
 };
 
 /* Conversion tables for PC characters */
@@ -388,102 +579,153 @@ unsigned char pc_to_iso1[256]={
 /* 250 */ 183,185,179,178,SP ,SP };
 
 
-
-
 /*** Global functions ***/
 
-GSList *get_contact_from_clist( void )
+
+#ifdef TRACE_FUNCTION
+void
+ICUTrace(const gchar *format, ...)
 {
-    GtkCList *clist = GTK_CLIST( MainData->lb_userwin );
-    g_return_val_if_fail( GTK_IS_WIDGET( clist ), NULL );
+  va_list args;
+  gchar *str;
 
-    if( clist->selection == NULL )
-	    return NULL;
+  g_return_if_fail (format != NULL);
 
-    return Find_User( GPOINTER_TO_INT( gtk_clist_get_row_data( clist, GPOINTER_TO_INT( clist->selection->data ) ) ) );
+  va_start (args, format);
+  str = g_strdup_vprintf (format, args);
+  va_end (args);
+
+  fputs ("gnomeicu trace: ", stderr);
+  fputs (str, stderr);
+
+  g_free (str);
 }
+#endif
 
-void select_clist_row_from_contact( CONTACT_PTR contact )
-{
-    GtkWidget *clist;
-
-    clist = GTK_WIDGET ( Which_List( contact ) );
-
-    MainData->lb_userwin = clist;
-    gtk_clist_select_row( GTK_CLIST( clist ), contact->lb_index, 0 );
-}
-
-GList *get_time_zones( void )
+GList *get_codes( FIELD_T field )
 {
 	GList *l = NULL;
-	int cx;
+	int i;
+	WORD last = 0xffff;
+	FIELD_CODE *array;
 
-	for( cx = 0; time_zones[ cx ].gmt_offset != 100; cx ++ )
-		l = g_list_prepend( l, _(time_zones[ cx ].str) );
-	l = g_list_reverse( l );
-	
-	return l;
-}
-
-GList *get_country_codes( void )
-{
-  GList *l = NULL;
-  gint cx;
-  
-  for( cx = 0; Country_Codes[ cx ].code != 0xffff; cx ++ )
-    l = g_list_prepend( l, _(Country_Codes[ cx ].name) );
-  l = g_list_sort( l, (GCompareFunc) strcoll );
-  
-  return l;
-}
-
-const gchar *Get_Country_Name( int code )
-{
-   gint i;
-   
-   for ( i = 0; Country_Codes[i].code != 0xffff; i++)
-     if ( Country_Codes[i].code == code )
-       return _(Country_Codes[i].name);
-
-   return _(Country_Codes[i].name);
-}
-
-const gchar *Get_Time_Zone_String( char code )
-{
-	int cx;
-
-	for( cx = 0; time_zones[ cx ].gmt_offset != 100; cx ++ )
-	if( time_zones[ cx ].gmt_offset == code )
-		return _(time_zones[ cx ].str);
-
-	return _(time_zones[ cx ].str);
-}
-
-int Get_Country_Code( const gchar *country )
-{
-	int cx;
-
-	for( cx = 0; Country_Codes[ cx ].code != 0xffff; cx++ )
-	  if( !strcmp( _(Country_Codes[ cx ].name), country ) )
-	    return Country_Codes[ cx ].code;
-
-	return 0;
-}
-
-char Get_Time_Zone( const gchar *desc )
-{
-	int cx;
-
-	for( cx = 0; time_zones[ cx ].gmt_offset != 100; cx ++ )
-	{
-		if( !strcmp( _(time_zones[ cx ].str), desc ) )
-		{
-			return time_zones[ cx ].gmt_offset;
-		}
+	switch (field) {
+		case TIMEZONE:
+			last = 100;
+			array = time_zones;
+			break;
+		case COUNTRY:
+			array = Country_Codes;	
+			break;
+		case INTEREST:
+			last = 255;
+			array = interest_codes;	
+			break;
+		case LANGUAGE:
+			last = 255;
+			array = language_codes;	
+			break;
+		case PAST:
+			array = past_codes;	
+			break;
+		case WORK:
+			array = work_codes;	
+			break;
+		case AFFILIATION:
+			array = affiliation_codes;	
+			break;
+		default:
+			return NULL;
 	}
-	if( !strcmp( _(time_zones[ cx ].str), desc ) )
-		return time_zones[ cx ].gmt_offset;
-	return 0;
+
+	for( i = 0; array[i].code != last; i++ )
+		l = g_list_prepend( l, _(array[i].name) );
+	
+	return g_list_reverse( l );
+}
+
+gchar *get_name( FIELD_T field, WORD code )
+{
+	int i;
+	WORD last = 0xffff;
+	FIELD_CODE *array;
+
+	switch (field) {
+		case TIMEZONE:
+			last = 100;
+			array = time_zones;
+			break;
+		case COUNTRY:
+			array = Country_Codes;	
+			break;
+		case INTEREST:
+			last = 255;
+			array = interest_codes;	
+			break;
+		case LANGUAGE:
+			last = 255;
+			array = language_codes;	
+			break;
+		case PAST:
+			array = past_codes;	
+			break;
+		case WORK:
+			array = work_codes;	
+			break;
+		case AFFILIATION:
+			array = affiliation_codes;	
+			break;
+		default:
+			return NULL;
+	}
+
+	for( i = 0; array[i].code != last; i++ )
+		if ( array[i].code == code )
+			return _(array[i].name);
+
+	return _(array[i].name);
+}
+
+WORD get_code( FIELD_T field, const gchar *name )
+{
+	int i;
+	WORD last = 0xffff;
+	FIELD_CODE *array;
+
+	switch (field) {
+		case TIMEZONE:
+			last = 100;
+			array = time_zones;
+			break;
+		case COUNTRY:
+			array = Country_Codes;	
+			break;
+		case INTEREST:
+			last = 255;
+			array = interest_codes;	
+			break;
+		case LANGUAGE:
+			last = 255;
+			array = language_codes;	
+			break;
+		case PAST:
+			array = past_codes;	
+			break;
+		case WORK:
+			array = work_codes;	
+			break;
+		case AFFILIATION:
+			array = affiliation_codes;	
+			break;
+		default:
+			return 0xffff; /* err... 0xffff (for equivalent) seems to be the best */
+	}
+
+	for( i = 0; array[i].code != last; i++ )
+		if( !strcmp( _(array[i].name), name ) )
+			return array[i].code;
+
+	return array[i].code;
 }
 
 const gchar *get_status_str( DWORD status )
@@ -511,31 +753,6 @@ const gchar *get_status_str( DWORD status )
 	}
 }
 
-/*******************************************************
-Gets config info from the rc file in the users home 
-directory.
-********************************************************/
-void Get_Unix_Config_Info( void )
-{
-	gchar *tmp;
-
-#ifdef TRACE_FUNCTION
-	g_print( "Get_Unix_Config_Info\n" );
-#endif
-
-	gnome_config_push_prefix( configfilename );
-
-	if ( (tmp=gnome_config_get_string( "Existence/exists" )) == NULL )
-		Initialize_RC_File();
-
-	g_free(tmp);
-
-	gnome_config_pop_prefix();
-
-	Read_RC_File();
-}
-
-
 /********************************************
 Converts an ip number character sequence to
 a DWORD
@@ -629,18 +846,56 @@ void Word_2_Chars( guchar buf[], WORD num )
 }
 
 /* Strip '\r' from text */
-gchar *stripr( const gchar *str )
+gchar *strip_cr( const gchar *str )
 {
 	gchar *new;
-	gint cx = 0, cy = 0;
+	gint cy = 0;
+
+	if (str == NULL)
+		return NULL;
 
 	new = (gchar*)g_malloc0( strlen( str ) + 1 );
 
-	while( str[cx] )
+	while (*str != '\0')
 	{
-		if( str[cx] != '\r' )
-			new[cy++] = str[cx];
-		cx++;
+		if( *str != '\r' ) {
+			new[cy] = *str;
+			cy++;
+		}
+		str++;
+	}
+
+	return new;
+}
+
+/* Strip HTML out of AIM messages */
+gchar *strip_html( const gchar *str )
+{
+	gchar *new;
+	gint cy = 0;
+	gboolean addchar = FALSE;
+
+	if (str == NULL)
+		return NULL;
+
+	new = (gchar*)g_malloc0( strlen( str ) + 1 );
+
+	if (!strncmp (str, "<HTML>", 6)) {
+		while (*str != '\0')
+		{
+			if( *str == '<' )
+				addchar = FALSE;
+
+			if (addchar == TRUE) {
+				new[cy] = *str;
+				cy++;
+			}
+			if (*str == '>')
+				addchar = TRUE;
+			str++;
+		}
+	} else {
+		return g_strdup(str);
 	}
 
 	return new;
@@ -648,42 +903,54 @@ gchar *stripr( const gchar *str )
 
 gchar *add_cr( const gchar *text )
 {
-        char *t;
-        int cx, cy;
+	const gchar* tmp = text;
+	gchar* res;
+	int cy;
 
-        cy = 0;
-        for( cx = 0; cx < strlen( text ); cx ++ )
-        {
-                if( text[ cx ] == '\n' )
-                        cy ++;
-        }
+	if (text == NULL)
+		return NULL;
 
-        t = g_malloc( strlen( text ) + cy + 1 );
+	cy = 0;
 
-        cy = 0;
-        for( cx = 0; cx < strlen( text ); cx ++ )
-        {
-                if( text[ cx ] != '\n' )
-                        t[ cy ++ ] = text[ cx ];
-                else
-                {
-                        t[ cy ++ ] = '\r';
-                        t[ cy ++ ] = '\n';
-                }
-        }
+	while (*tmp != '\0')
+	{
+		if (*tmp == '\n') {
+			cy ++;
+		}
+		tmp++;
+	}
 
-        t[ cy ] = 0x00;
+	res = g_malloc( strlen( text ) + cy + 1 );
 
-        return t;
+	cy = 0;
+	tmp = text;
+	while (*tmp != '\0')
+	{
+		if( *tmp != '\n' ) {
+			res[cy] = *tmp;
+			cy++;
+		} else {
+			res[cy] = '\r';
+			res[cy+1] = '\n';
+			cy+=2;
+		}
+		tmp++;
+	}
+	res[cy] = '\0';
+
+	return res;
 }
 
 
 GSList * Find_User( UIN_T uin )
 {
 	GSList *contact = Contacts;
+	if (uin == NULL)
+		return NULL;
+
 	while( contact != NULL )
 	{
-		if( kontakt->uin == uin )
+		if( !strcmp (kontakt->uin, uin) )
 			break;
 		contact = contact->next;
 	}
@@ -715,42 +982,42 @@ GSList * Add_User( UIN_T uin, const gchar *name, gboolean inlist )
 	g_print( "Add_User\n" );
 #endif
 
-	contact = Find_User( uin );
+	contact = Find_User (uin);
+	while (contact && kontakt->gid == 0) {
+		Contacts = g_slist_remove (Contacts, contact->data);
+		contact = Find_User (uin);
+	}
+
 	if( contact != NULL )
 		return contact;
 
 	contactdata = g_new0( Contact_Member, 1 );
 
-	contactdata->uin = uin;
-	contactdata->icon_p = icon_offline_pixmap;
-	contactdata->icon_b = icon_offline_bitmap;
+	contactdata->uin = g_strdup (uin);
 	contactdata->stored_messages = NULL;
 	contactdata->status = STATUS_OFFLINE;
-	/* The following two is a hack for Show_Quick_Status_lower to add to
-	 * it's OfflineList */
-	contactdata->last_status = STATUS_NA;
-	contactdata->need_update = FALSE;
 	contactdata->last_time = -1L;
 	contactdata->current_ip = -1L;
         contactdata->has_direct_connect = FALSE;
 	contactdata->port = 0;
 	contactdata->sok = 0;
 	contactdata->invis_list = FALSE;
-	contactdata->chat_requests = NULL;
 	contactdata->msg_dlg_xml = NULL;
 	contactdata->confirmed = inlist; /* Needs confirmation if not manually added*/
 
-	if( uin == 10 )
-		strncpy( contactdata->nick, _("Web Message"), 20 );
-	else
-	{
-		if( name == NULL || name[0] == '\0' )
-			g_snprintf( contactdata->nick, 20, "%d", uin );
-		else
-			strncpy( contactdata->nick, name, 20 );
+	if( !strcmp (uin, "10") ) {
+		g_free(contactdata->nick);
+		contactdata->nick = g_strdup(_("Web Message"));
+	} else {
+		if( name == NULL || name[0] == '\0' ) {
+			g_free(contactdata->nick);
+			contactdata->nick = g_strdup(uin);
+		} else {
+			g_free(contactdata->nick);
+			contactdata->nick = g_strdup(name);
+		}
 	}
 
-	contactdata->show_again = TRUE;
 	contactdata->info = g_new0( USER_INFO_STRUCT, 1 );
 	contactdata->info->sex = NOT_SPECIFIED;
 	contactdata->info->country = 1;
@@ -761,25 +1028,22 @@ GSList * Add_User( UIN_T uin, const gchar *name, gboolean inlist )
 
 	/* TODO: insert sorted */
 	Contacts = g_slist_append( Contacts, contactdata );
+	contact = g_slist_last (Contacts);
 
-        if (!inlist) {
+        if (!inlist && mainconnection && Current_Status != STATUS_OFFLINE) {
           GSList *requestdat;
           DWORD reqid;
 
-          reqid = v7_request_info(uin);
+          reqid = v7_request_info(mainconnection, uin);
           
           requestdat =  v7_get_request_by_reqid(reqid);
           ((Request*)requestdat->data)->type = NEW_USER_VERIFICATION;
         }
 
-        /* v7_addcontact(uin); */
-
-/*	Show_Quick_Status_lower( inlist ? UPDATE_OFFLINE : UPDATE_NOTLIST, NULL );*/
-
 	Save_RC();
 	
 	/* TODO: and find it */
-	return g_slist_last( Contacts );
+	return contact;
 }
 
 /********************************************
@@ -838,104 +1102,74 @@ void Word_2_CharsBE( guchar buf[], WORD num )
 	buf[1] = ( guchar ) ((unsigned)num) & 0x00FF;
 }
 
-gchar *dertf(gchar *rtfdata)
+
+gchar *convert_to_utf8(const gchar *text)
 {
+  gchar *converted_text;
+  GError *converterror = NULL;
 
-  gchar *finaltext;
-  int i=1, j=0, crlf=2;
-  int charset = 0;
+  converted_text = g_convert_with_fallback(text, strlen(text)+1, 
+					   "UTF-8", 
+		         preferences_get_string(PREFS_GNOMEICU_CHARSET_FALLBACK),
 
-#ifdef TRACE_FUNCTION
-  g_print( "dertf\n" );
-#endif
-  
+					   "?", NULL, NULL, &converterror);
 
-  finaltext = g_malloc0(strlen(rtfdata)+1);
+  if (converterror)
+    g_warning("Conversing error: %s\n", converterror->message);
 
-  if (strncmp(rtfdata, "{\\rtf", 5) != 0) {
-    strcpy(finaltext, rtfdata);
-    return finaltext;
-  }
-  
-  /* g_print("Origlen: %d\n", strlen(rtfdata)); */
-  
-  for(; i < strlen(rtfdata); i++) {
-    /* g_print("Level1: %d %c\n", i, rtfdata[i]); */
-    if (rtfdata[i] == '\\') {
-      i++;
+  g_clear_error(&converterror);
 
-      if (!strncmp(rtfdata+i, "fbcharset", 8)) {
-        i += 8;
-        charset = strtol(rtfdata+i, NULL, 10);
-      }
-      else if (rtfdata[i] == '\'' &&
-          isxdigit(rtfdata[i+1]) &&
-          isxdigit(rtfdata[i+2])) {
-        char blah[3] = "\0\0\0";
-        int value;
-        blah[0] =  rtfdata[++i];
-        blah[1] =  rtfdata[++i];
-        value = strtol(blah, NULL, 16);
-        /* g_print("Acc char: %d %d  .. %c %c\n", value, value, 
-           pc_to_iso1[value], pc_to_iso1[value]); */
-        if (charset == 255)
-          finaltext[j++] = (value>127) ? pc_to_iso1[value] : value;
-        else 
-          finaltext[j++] = value;
-      }
-      
-      else {
-        for (; i<strlen(rtfdata);i++) {
-          /* g_print("Level2: %d %c\n", i, rtfdata[i]); */
-          if (isalnum(rtfdata[i]) || rtfdata[i] == '-' || rtfdata[i] == '{') {
-            if (rtfdata[i] == '{') {
-              int par=0;
-              for(; i < strlen(rtfdata); i++) {
-                /* g_print("Level3: %d %d %c\n", par, i, rtfdata[i]); */
-                if (rtfdata[i] == '{')
-                  par++;
-                if (rtfdata[i] == '}')
-                  par--;
-                if (par == 0)
-                  break;
-              }
-              i--;
-            }
-          } else {
-            if (! isspace(rtfdata[i]))
-              i--;
-            break;
-          }
-        }
-      }
-    } else if (rtfdata[i] == '{') {
-      int par=0;
-      for(; i < strlen(rtfdata); i++) {
-        /* g_print("Level4: %d %d %c\n", i, par, rtfdata[i]); */
-        if (rtfdata[i] == '{')
-          par++;
-        if (rtfdata[i] == '}')
-          par--;
-        if (par == 0)
-          break;
-      }
-      i--;
-    }
-    else if (rtfdata[i] == '}')
-      continue;
-    else if (rtfdata[i] == '\r') {
-      i++;
-      if (crlf) 
-        crlf--;
-      else
-        finaltext[j++] = '\n';
-    }
-    else
-      finaltext[j++] = rtfdata[i];
-    
+  return converted_text;
+}
+
+gchar *convert_from_utf8(const gchar *text)
+{
+  gchar *converted_text;
+  GError *converterror = NULL;
+  
+  converted_text = g_convert_with_fallback(text, strlen(text)+1, 
+		         preferences_get_string(PREFS_GNOMEICU_CHARSET_FALLBACK),
+					   "UTF-8", 
+					   "?", NULL, NULL, &converterror);
+  if (converterror)
+    g_warning("Conversing error: %s\n", converterror->message);
+  
+  g_clear_error(&converterror);
+
+  return converted_text;
+
+}
+
+GladeXML *
+gicu_util_open_glade_xml (const gchar *glade_file, const gchar *root_name)
+{
+  GladeXML *gxml;
+  gchar *gfile;
+
+  g_assert (glade_file != NULL);
+  g_assert (root_name != NULL);
+
+  gfile = g_strdup_printf ("%s%s", GNOMEICU_GLADEDIR, glade_file);
+
+  gxml = glade_xml_new (gfile, root_name, NULL);
+  if (!gxml) {
+    GtkWidget *dialog;
+    gchar *err_str;
+
+    err_str = g_strdup_printf (_("Unable to load Glade user interface file: "
+                                 "%s.\nMake sure the file is accessible."),
+                               gfile);
+
+    dialog = gtk_message_dialog_new (GTK_WINDOW (MainData->window),
+                                     GTK_DIALOG_DESTROY_WITH_PARENT,
+                                     GTK_MESSAGE_ERROR, GTK_BUTTONS_OK, err_str);
+    gtk_dialog_run (GTK_DIALOG (dialog));
+
+    gtk_widget_destroy (dialog);
+    g_free (err_str);
   }
-  
-  finaltext[j-2] = 0;
+
+  g_free (gfile);
 
-  return finaltext;
+  return gxml;
 }
