@@ -1,6 +1,6 @@
 /****************************
   Reception for ICQ v6-7 protocol (Oscar)
-  Olivier Crete (c) 2001
+  Olivier Crete (c) 2001-2002
   GnomeICU
 *****************************/
 
@@ -9,8 +9,7 @@
 
 #include "v7base.h"
 
-void v7_connect(gchar *address, guint port, gchar *cookie, guint cookielen);
-void v7_reqofflinemsg (void);
+V7Connection *v7_connect(gchar *address, guint port, gchar *cookie, guint cookielen);
 
 #endif /* __V7RECV_H__ */
 
